(function() {
    if (typeof HKQueue !== "undefined") {
        return;
    }

    var HKMAX_TASK_COUNT = 20;
    var HKTask = function(taskId, onSuccess, onFailure) {
        this.id = taskId;
        this.successCallback = onSuccess;
        this.failureCallback = onFailure;
    };
    var HKQueue = {
        tasks: [],
        taskCount: 0,
        createTask: function(onSuccess, onFailure) {
            var taskId = this.taskCount;
            var index = taskId % HKMAX_TASK_COUNT;
            var task = new HKTask(taskId, onSuccess, onFailure);
            this.tasks[index] = task;
            this.taskCount += 1;
            return task;
        },
        getTask: function(taskId) {
            var index = taskId % HKMAX_TASK_COUNT;
            return this.tasks[index];
        }
    };
    window.HKFireTask = function(taskId, data) {
        var task = HKQueue.getTask(taskId);
        if (task && task.id === taskId && typeof task.successCallback === "function") {
            task.successCallback(data);
        }
    };
    window.HKOnError = function (taskId, errmsg) {
        var task = HKQueue.getTask(taskId);
        if (task && task.id === taskId && typeof task.failureCallback === "function") {
            task.failureCallback(errmsg);
        }
    };


    // 注册事件
    var registerHandler = function(method) {
        var arr = method.split('.', 2);
        if (arr.length != 2) {
            return;
        }
        var className = arr[0].trim();
        var functionName = arr[1].trim();
        var paramKeys = Array.prototype.slice.call(arguments, 1);

        var createMessage = function(argv) {
            var task;
            var paramValues; // 除去回调的参数
            var data = {};
            var i;

            // 生成task、参数值
            var argc = argv.length;
            var calbackCount = (typeof argv[argc-1] === "function") + (typeof argv[argc-1] === "function" && typeof argv[argc-2] === "function");
            task = HKQueue.createTask.apply(HKQueue, argv.slice(argc-calbackCount));
            paramValues = argv.slice(0, argc-calbackCount);

            // 生成参数
            for (i = 0; i < paramValues.length; i++) {
                if (paramKeys[i]) {
                    data[paramKeys[i]] = paramValues[i];
                } else if (typeof paramValues[i] === "object") {
                    obj_extend(data, paramValues[i]);
                }
            }

            // 返回消息
            return {
                className: className + "Bridge",
                functionName: functionName,
                taskId:task.id,
                data: data
            };
        };

        window[className] = window[className] || {};
        window[className][functionName] = function() {
            var msg = createMessage(Array.prototype.slice.call(arguments, 0));
            if (window.androidBridge) {
                window.androidBridge.jsCallnative(JSON.stringify(msg));
            } else {
                window.webkit.messageHandlers.nybHandler.postMessage(msg);
            }
        };
    };
    // 合并b中的属性到a
    var obj_extend = function(a, b) {
        var key;
        for(key in b) {
            if (b.hasOwnProperty(key)) {
                a[key] = b[key];
            }
        }
    };


    // API请求
    registerHandler("HKRequest.GET", "url", "params");
    registerHandler("HKRequest.POST", "url", "params");

    // 用户
    registerHandler("HKUser.whoami"); // 登录状态回调函数参数: { memberId: 123, memberType: "member", nickname: "xxx", }
    registerHandler("HKUser.checkLogin");
    registerHandler("HKUser.resetPoints", "points");


    // 网页菜单
    registerHandler("HKWebView.showMenu");
    registerHandler("HKWebView.showScrollsToTop", "isShow");

    // 事件
    registerHandler("HKEvent.countEvent", "eventId", "params");

    // 订单
    registerHandler("HKOrder.pay", "type", "params"); // type: 1=>微信，2=>支付宝

    // 原生功能
    registerHandler("HKNative.location");
    registerHandler("HKNative.notification", "name", "data"); // 给原生发通知


    registerHandler("HKMedia.chooseImage");
    registerHandler("HKMedia.previewImage");
    registerHandler("HKMedia.uploadImage");
    registerHandler("HKMedia.getLocalImgData");


    registerHandler("HKWidget.picker");
    registerHandler("HKWidget.select");
         if (window.androidBridge) {
                window.androidBridge.jsCallnative("injected");
                        }

})();